"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var connection_1 = require("./connection");
var logger_1 = __importDefault(require("./logger"));
var State;
(function (State) {
    State["INIT"] = "initialized";
    State["CONNECTED"] = "connected";
    State["ERROR"] = "error";
    State["CLOSED"] = "closed";
    State["CONNECTING"] = "connecting";
    State["DISCONNECTED"] = "disconnected";
})(State = exports.State || (exports.State = {}));
var EmitterEvent;
(function (EmitterEvent) {
    EmitterEvent["INIT"] = "initialized";
    EmitterEvent["CONNECTED"] = "connected";
    EmitterEvent["ERROR"] = "error";
    EmitterEvent["CLOSED"] = "closed";
    EmitterEvent["ATCLOSED"] = "@closed";
    EmitterEvent["CONNECTING"] = "connecting";
    EmitterEvent["DISCONNECTED"] = "disconnected";
    EmitterEvent["MSG"] = "message";
    EmitterEvent["RETRYMAX"] = "retryMax";
    EmitterEvent["RETRY"] = "retry";
})(EmitterEvent = exports.EmitterEvent || (exports.EmitterEvent = {}));
var ConnectionManager = /** @class */ (function () {
    function ConnectionManager(key, options) {
        var _this = this;
        this.key = key;
        this.options = options;
        this.state = State.INIT;
        this.url = options.url;
        this.token = options.token;
        this.emitter = new events_1.EventEmitter();
        this.reconnection = !!options.reconnection;
        this.reconnectionDelay = !options.reconnectionDelay
            ? 3000
            : options.reconnectionDelay;
        this.retryMax =
            options.retryMax === undefined
                ? Number.MAX_SAFE_INTEGER
                : options.retryMax;
        this.skipReconnect = false;
        this.retryNum = 0;
        this.connectionStartTimestamp = 0;
        this.retryTimer = null;
        this.connection = new connection_1.Connection({ url: this.url, token: this.token });
        this.connection.bind(connection_1.EmitterEvent.OPEN, function () {
            _this.connectionStartTimestamp = Date.now();
            if (_this.retryTimer) {
                clearTimeout(_this.retryTimer);
                _this.retryNum = 0;
                _this.retryTimer = null;
            }
            _this.skipReconnect = false;
            _this.updateState(State.CONNECTED);
        });
        this.connection.bind(connection_1.EmitterEvent.MSG, function (message) {
            _this.emitter.emit(EmitterEvent.MSG, message);
        });
        this.connection.bind(connection_1.EmitterEvent.ERROR, function (err) {
            _this.updateState(State.ERROR, err);
        });
        this.connection.bind(connection_1.EmitterEvent.CLOSED, function (evt) {
            var sessionTime = Date.now() - _this.connectionStartTimestamp;
            if (sessionTime > 0 && _this.connectionStartTimestamp !== 0) {
                _this.emitter.emit("@closed", Object.assign({}, evt, { session_time: sessionTime }));
                logger_1.default.log("Session Time: " + sessionTime + " ms");
                _this.connectionStartTimestamp = 0;
            }
            _this.updateState(State.CLOSED, evt);
            _this.retryIn(_this.reconnectionDelay);
        });
    }
    ConnectionManager.prototype.retryIn = function (delay) {
        var _this = this;
        if (delay === void 0) { delay = 0; }
        if (this.retryNum >= this.retryMax) {
            this.disconnect();
            this.emitter.emit(EmitterEvent.RETRYMAX);
            logger_1.default.log("Reconnect Max: ", this.retryNum);
        }
        if (this.reconnection && !this.skipReconnect) {
            this.retryTimer = window.setTimeout(function () {
                _this.retryNum += 1;
                logger_1.default.log("Reconnect attempts: ", _this.retryNum);
                _this.connect();
                _this.emitter.emit(EmitterEvent.RETRY, { retry: _this.retryNum });
            }, delay);
        }
    };
    ConnectionManager.prototype.bind = function (event, callback) {
        this.emitter.on(event, callback);
        return this;
    };
    ConnectionManager.prototype.unbind = function (event, callback) {
        this.emitter.removeListener(event, callback);
        return this;
    };
    ConnectionManager.prototype.unBindAll = function () {
        var evt = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            evt[_i] = arguments[_i];
        }
        if (evt.length) {
            this.emitter.removeAllListeners(evt);
        }
        else {
            this.emitter.removeAllListeners();
        }
    };
    ConnectionManager.prototype.connect = function () {
        this.updateState(State.CONNECTING);
        this.connection.connect(this.token);
        logger_1.default.log("Auth", {
            token: this.token,
        });
    };
    ConnectionManager.prototype.disconnect = function () {
        this.skipReconnect = true;
        this.connection.close();
        this.updateState(State.DISCONNECTED);
    };
    ConnectionManager.prototype.updateState = function (newState, data) {
        var previousState = this.state;
        this.state = newState;
        if (previousState !== newState) {
            logger_1.default.log("State changed", "'" + previousState + "' -> '" + newState + "'");
            this.emitter.emit(newState, data);
        }
    };
    ConnectionManager.prototype.send = function (event, data, channel) {
        if (this.connection) {
            return this.connection.send(event, data, channel);
        }
        return false;
    };
    return ConnectionManager;
}());
exports.ConnectionManager = ConnectionManager;
