"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var lodash_chunk_1 = __importDefault(require("lodash.chunk"));
var channels_1 = require("./channels");
var connection_manager_1 = require("./connection-manager");
var system_1 = require("./system");
var logger_1 = __importDefault(require("./logger"));
var GusherEvent;
(function (GusherEvent) {
    GusherEvent["ALL"] = "*";
    GusherEvent["CONNECTED"] = "connected";
    GusherEvent["DISCONNECTED"] = "disconnected";
    GusherEvent["RETRY"] = "retry";
    GusherEvent["RETRYMAX"] = "retryMax";
    GusherEvent["CLOSED"] = "closed";
    GusherEvent["ATCLOASED"] = "@closed";
    GusherEvent["ERROR"] = "error";
})(GusherEvent || (GusherEvent = {}));
var Gusher = /** @class */ (function () {
    function Gusher(appKey, options) {
        if (appKey === void 0) { appKey = ""; }
        this.key = appKey;
        this.options = options;
        this.emitter = new events_1.EventEmitter();
        this.channels = new channels_1.Channels();
        this.connection = this.createConnection();
    }
    Gusher.prototype.unsubscribe = function (channelName) {
        var channel = this.channels.remove(channelName);
        if (channel && this.connection.state === "connected") {
            channel.unsubscribe();
        }
    };
    Gusher.prototype.getAuthToken = function () {
        return this.options.token || "";
    };
    Gusher.prototype.setAuthToken = function (token) {
        if (!token) {
            return;
        }
        this.options.token = token;
        this.connection.unBindAll();
        this.connection.disconnect();
        this.connection = this.createConnection();
    };
    Gusher.prototype.createConnection = function () {
        var _this = this;
        var connection = new connection_manager_1.ConnectionManager(this.key, this.options);
        connection.bind(connection_manager_1.EmitterEvent.CONNECTED, function () {
            _this.subscribeAll();
            _this.emitter.emit(GusherEvent.CONNECTED);
        });
        connection.bind(connection_manager_1.EmitterEvent.MSG, function (params) {
            if (!params) {
                return;
            }
            if (params.channel) {
                var channel = _this.channels.find(params.channel);
                if (channel) {
                    channel.handleEvent(params.event, params.data);
                }
            }
            if (params.data &&
                params.data.channel &&
                Array.isArray(params.data.channel)) {
                params.data.channel.forEach(function (ch) {
                    var channel = _this.channels.find(ch);
                    if (channel) {
                        channel.handleEvent(params.event, params.data);
                    }
                });
            }
            _this.emitter.emit(params.event, params.data);
            _this.emitter.emit(GusherEvent.ALL, params);
        });
        connection.bind(connection_manager_1.EmitterEvent.DISCONNECTED, function () {
            _this.channels.disconnect();
            _this.emitter.emit(GusherEvent.DISCONNECTED);
        });
        connection.bind(connection_manager_1.EmitterEvent.RETRY, function (evt) {
            _this.emitter.emit(GusherEvent.RETRY, evt);
        });
        connection.bind(connection_manager_1.EmitterEvent.RETRYMAX, function () {
            _this.emitter.emit(GusherEvent.RETRYMAX);
        });
        connection.bind(connection_manager_1.EmitterEvent.ATCLOSED, function (evt) {
            _this.emitter.emit(GusherEvent.ATCLOASED, evt);
        });
        connection.bind(connection_manager_1.EmitterEvent.CLOSED, function (evt) {
            _this.emitter.emit(GusherEvent.CLOSED, evt);
        });
        connection.bind(connection_manager_1.EmitterEvent.ERROR, function (err) {
            logger_1.default.log("Error", err);
            _this.emitter.emit(GusherEvent.ERROR, err);
        });
        return connection;
    };
    Gusher.prototype.channel = function (name) {
        return this.channels.find(name);
    };
    Gusher.prototype.allChannel = function () {
        return this.channels.all();
    };
    Gusher.prototype.connect = function () {
        this.connection.connect();
    };
    Gusher.prototype.disconnect = function () {
        this.connection.disconnect();
    };
    Gusher.prototype.bind = function (event, callback) {
        this.emitter.on(event, callback);
        return this;
    };
    Gusher.prototype.unbind = function (event, callback) {
        this.emitter.removeListener(event, callback);
        return this;
    };
    Gusher.prototype.subscribe = function (channelName) {
        var channel = this.channels.add(channelName, this);
        if (this.connection.state === "connected") {
            channel.subscribe();
        }
        return channel;
    };
    Gusher.prototype.subscribes = function (channels) {
        var _this = this;
        var returnValues = {};
        if (!Array.isArray(channels)) {
            return returnValues;
        }
        channels.forEach(function (channelName) {
            returnValues[channelName] = _this.channels.add(channelName, _this);
        });
        if (this.connection.state === "connected") {
            this.subscribeAll(channels);
        }
        return returnValues;
    };
    Gusher.prototype.subscribeAll = function (channels) {
        var _this = this;
        var multiChannel = channels || this.channels.all();
        lodash_chunk_1.default(multiChannel, 10).forEach(function (group) {
            _this.send(system_1.Action.MULTI_SUBSCRIBE, { multi_channel: group });
        });
    };
    Gusher.prototype.send = function (event, data, channel) {
        this.connection.send(event, data, channel);
    };
    Gusher.Event = GusherEvent;
    return Gusher;
}());
exports.Gusher = Gusher;
